@@ -75,54 +75,67 @@ def run_forecast_ensemble(
         print(f"    Running --{model_name}")
         model_dict = {"name": model_name}
         model = initialize_model(model_name)
-        print("Initialized")
-        results = model.evaluate(
+        model_dict = model.evaluate(
             dataframe=df,
             train_end_date=train_end_date,
             frequency=frequency,
             forecast_period_list=period_list)
-        model_dict["train_timestamps"] = results.pop("train_timestamps")
-        model_dict["prediction_values"] = results.pop("prediction_values")
-        model_dict["actual_values"] = results.pop("actual_values")
-        model_dict["forecast_timestamps"] = results.pop("forecast_timestamps")
-        model_dict["forecast_values"] = results.pop("forecast_values")
-        model_dict["performance_metrics"] = results
-        weight = model_dict["performance_metrics"]["r2"]
+        weight = model_dict["r2"]
         if weight < 0:
             weight = 0
-        elif model["name"] == "Naive":
+        elif model_name == "Naive":
             weight = 0
         model_dict["weight"] = weight
         ensemble.append(model_dict)
 
     # Combine outputs to calculate ensemble effectiveness
+    print("Creating Ensemble")
     total_weight = sum([model["weight"] for model in ensemble])
-    ensemble_predictions_list = []
-    ensemble_forecast_list = []
+    if total_weight == 0:
+        total_weight = 1
+    return_dataframe = df.copy().set_index(DATE_COLUMN_NAME)
     for model in ensemble:
         model["weight"] = model["weight"] / total_weight
+        return_dataframe = pd.merge(
+            left=return_dataframe,
+            right=model["forecast_dataframe"],
+            how="outer",
+            left_index=True,
+            right_index=True)
         if model["weight"] > 0:
-            predictions = pd.Series(model["prediction_values"]).reset_index(drop=True) * model["weight"]
-            forecast = pd.Series(model["forecast_values"]).reset_index(drop=True) * model["weight"]
-            actuals = pd.Series(model["actual_values"])
-            ensemble_predictions_list.append(predictions)
-            ensemble_forecast_list.append(forecast)
-    ensemble_train_timestamps = model["train_timestamps"]
-    ensemble_predictions = sum(ensemble_predictions_list)
-    ensemble_actuals = model["actual_values"]
-    ensemble_forecast_timestamps = model["forecast_timestamps"]
-    ensemble_forecast = sum(ensemble_forecast_list)
-
-    ensemble_train_df = pd.DataFrame({
-        "timestamps": ensemble_train_timestamps,
-        "actuals": ensemble_actuals,
-        "predictions": ensemble_predictions})
-    ensemble_forecast_df = pd.DataFrame({
-        "timestamps": ensemble_forecast_timestamps,
-        "forecast": ensemble_forecast})
-    ensemble_train_df = ensemble_train_df[~ensemble_train_df.predictions.isnull()]
-    performance_metrics = utils.get_model_statistics(ensemble_train_df["predictions"], ensemble_train_df["actuals"])
-    consolidated_metrics = utils.consolidate_scores(performance_metrics, ensemble_train_df["actuals"].mean())
+            train_df = model["train_dataframe"]
+            train_df[f"weighted_predicted_values_{model['name']}"] = train_df[f"predicted_values_{model['name']}"] * model["weight"]
+            return_dataframe = pd.merge(
+                left=return_dataframe,
+                right=model["train_dataframe"],
+                how="outer",
+                left_index=True,
+                right_index=True)
+            forecast_df = model["forecast_dataframe"]
+            forecast_df[f"forecast_values_{model['name']}"] = forecast_df[f"forecast_values_{model['name']}"] * model["weight"]
+            forecast_df.rename(columns={f"forecast_values_{model['name']}": f"weighted_forecast_values_{model['name']}"}, inplace=True)
+            return_dataframe = pd.merge(
+                left=return_dataframe,
+                right=forecast_df[[f"weighted_forecast_values_{model['name']}"]],
+                how="outer",
+                left_index=True,
+                right_index=True)
+    # Create Ensemble Predictions
+    ensemble_prediction_columns = [column for column in return_dataframe.columns if column.startswith("weighted_predicted_values")]
+    ensemble_prediction_eval = " + ".join(ensemble_prediction_columns)
+    return_dataframe["predicted_values_Ensemble"] = return_dataframe.eval(ensemble_prediction_eval)
+    # Create ensemble weighted forecast
+    ensemble_forecast_columns = [column for column in return_dataframe.columns if column.startswith("weighted_forecast_values")]
+    ensemble_forecast_eval = " + ".join(ensemble_forecast_columns)
+    return_dataframe["forecast_values_Ensemble"] = return_dataframe.eval(ensemble_forecast_eval)
+    # Calculate ensemble metrics
+    ensemble_train_df = return_dataframe[~return_dataframe["predicted_values_Ensemble"].isnull()]
+    performance_metrics = utils.get_model_statistics(ensemble_train_df["predicted_values_Ensemble"], ensemble_train_df["y"])
+    consolidated_metrics = utils.consolidate_scores(performance_metrics, ensemble_train_df["y"].mean())
+    # Filter to only the required columns
+    forecast_columns = [column for column in return_dataframe.columns if column.startswith("forecast_values")]
+    return_columns = ["y"] + forecast_columns
+    return_dataframe = return_dataframe[return_columns]
 
     degraded_accuracies = {}
     training_years = (df[DATE_COLUMN_NAME].max() - df[DATE_COLUMN_NAME].min()).days / 365
@@ -140,8 +153,7 @@ def run_forecast_ensemble(
     ensemble.append({
         "name": "ensemble",
         "model": None,
-        "train_dataframe": ensemble_train_df,
-        "forecast_dataframe": ensemble_forecast_df,
+        "ensemble_dataframe": return_dataframe,
         "performance_metrics": performance_metrics,
         "consolidated_metrics": consolidated_metrics,
         "weight": None
