@@ -4,7 +4,8 @@ import pandas as pd
 from datetime import datetime
 from dateutil.relativedelta import relativedelta
 
-from greykite.framework.templates.autogen.forecast_config import ForecastConfig, MetadataParam, EvaluationPeriodParam, ComputationParam
+from fbprophet import Prophet
+from greykite.framework.templates.autogen.forecast_config import ForecastConfig, MetadataParam, EvaluationPeriodParam
 from greykite.framework.templates.forecaster import Forecaster
 from greykite.framework.templates.model_templates import ModelTemplateEnum
 from xgboost import XGBRegressor
@@ -45,11 +46,16 @@ class BaseModel:
             frequency=frequency,
             forecast_period_list=forecast_period_list)
         result_package = utils.get_model_statistics(y=actual, yhat=predicted)
-        result_package["train_timestamps"] = pd.Series(actual.index).dt.strftime("%Y-%m-%d").values
-        result_package["actual_values"] = actual.values
-        result_package["prediction_values"] = predicted
-        result_package["forecast_timestamps"] = pd.Series(forecast.index).dt.strftime("%Y-%m-%d").values
-        result_package["forecast_values"] = pd.Series(forecast).values
+        result_package["name"] = self.__name__
+        result_package["forecast_dataframe"] = pd.DataFrame(
+                index=pd.Series(forecast.index),
+                data={f"forecast_values_{self.__name__}": pd.Series(forecast).values})
+        result_package["train_dataframe"] = pd.DataFrame(
+                index=pd.Series(actual.index),
+                data={
+                    f"actual_values_{self.__name__}": actual.values,
+                    f"predicted_values_{self.__name__}": predicted
+                })
 
         return result_package
 
@@ -59,6 +65,7 @@ class GreyKite(BaseModel):
     def __init__(self):
         """Initializes greykite model"""
         super().__init__()
+        self.__name__ = "GreyKite"
         self.metadata = MetadataParam(
             time_col=DATE_COLUMN_NAME,
             value_col=VALUE_COLUMN_NAME)
@@ -105,9 +112,7 @@ class FBProphet(BaseModel):
     def __init__(self):
         """Initializes prophet model"""
         super().__init__()
-        self.metadata = MetadataParam(
-            time_col=DATE_COLUMN_NAME,
-            value_col=VALUE_COLUMN_NAME)
+        self.__name__ = "FBProphet"
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
         """Runs rolling prediction using the model to return actual and predicted values
@@ -118,32 +123,20 @@ class FBProphet(BaseModel):
         Returns:
             dataframes with actual and predicted values
         """
-        forecaster = Forecaster()
-        self.metadata.freq = frequency
-        forecast_horizon = len(forecast_period_list)
-        if forecast_horizon >= (len(dataframe) / 3):
-            test_horizon = (len(dataframe) / 3) - 1
-        else:
-            test_horizon = forecast_horizon
-        result = forecaster.run_forecast_config(  # result is also stored as `forecaster.forecast_result`.
-            df=dataframe,
-            config=ForecastConfig(
-                evaluation_period_param=EvaluationPeriodParam(test_horizon=test_horizon, cv_horizon=0),
-                model_template=ModelTemplateEnum.PROPHET.name,
-                coverage=0.95,
-                metadata_param=self.metadata,
-                forecast_horizon=len(forecast_period_list)
-            )
-        )
-        backtest = result.backtest.df_test[~result.backtest.df_test["actual"].isnull()]
-        backtest = backtest[backtest[DATE_COLUMN_NAME] > train_end]
-        backtest = backtest.set_index("period")
-        forecast_df = result.forecast.df
-        forecast_df = forecast_df.set_index("period")
-        mask = forecast_df["actual"].isnull()
-        forecast = forecast_df.loc[mask, "forecast"]
+        df = dataframe.rename(columns={DATE_COLUMN_NAME: "ds", VALUE_COLUMN_NAME: "y"})
 
-        return backtest["actual"], backtest["forecast"], forecast
+        model = Prophet()
+        model.fit(df, iter=10)
+
+        future = model.make_future_dataframe(periods=len(forecast_period_list), freq=frequency)
+        forecast = model.predict(future)
+        forecast = forecast.rename(columns={"ds": DATE_COLUMN_NAME}).set_index(DATE_COLUMN_NAME)
+
+        df_forecast = pd.merge(dataframe.set_index(DATE_COLUMN_NAME), forecast, how="outer", left_index=True, right_index=True)
+        df_train = df_forecast[~df_forecast[VALUE_COLUMN_NAME].isnull()]
+        df_forecast = df_forecast[df_forecast[VALUE_COLUMN_NAME].isnull()]
+
+        return df_train[VALUE_COLUMN_NAME], df_train["yhat"], df_forecast["yhat"]
 
 
 class Naive(BaseModel):
@@ -151,17 +144,17 @@ class Naive(BaseModel):
     def __init__(self):
         """Initializes naive model"""
         super().__init__()
+        self.__name__ = "Naive"
         self.growth_comparison_period_in_years = 1
         self.minimum_training_years = 2
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
-        dataframe_in = dataframe.set_index("period")
         # Determine training and testing periods to create test and training dataframes
         comparison_period_relativedelta = relativedelta(years=self.growth_comparison_period_in_years)
         train_mask = dataframe[DATE_COLUMN_NAME] <= train_end
         dataframe_test = dataframe[~train_mask]
 
-        def run_prediction(prediction_date):
+        def run_prediction(prediction_date, frequency):
             base_growth_period_start = prediction_date - (2 * comparison_period_relativedelta)
             comparison_growth_period_start = prediction_date - comparison_period_relativedelta
 
@@ -170,6 +163,7 @@ class Naive(BaseModel):
                 (dataframe[DATE_COLUMN_NAME] >= base_growth_period_start) & \
                 (dataframe[DATE_COLUMN_NAME] < comparison_growth_period_start)
             base_growth_period_sum = dataframe.loc[base_growth_period_mask, VALUE_COLUMN_NAME].sum()
+
             # Calculate comparison period totals
             comparison_growth_period_mask = \
                 (dataframe[DATE_COLUMN_NAME] >= comparison_growth_period_start) & \
@@ -179,6 +173,8 @@ class Naive(BaseModel):
             growth_rate = comparison_growth_period_sum / base_growth_period_sum
 
             # Determine value to use as the base for the estimation
+            if frequency == "W-MON":
+                comparison_growth_period_start = comparison_growth_period_start - relativedelta(days=(comparison_growth_period_start.weekday()))
             previous_value_mask = dataframe[DATE_COLUMN_NAME] == comparison_growth_period_start
             previous_value = dataframe.loc[previous_value_mask, VALUE_COLUMN_NAME].sum()
 
@@ -190,7 +186,7 @@ class Naive(BaseModel):
         for index, row in dataframe_test.iterrows():
             # Determine window to evaluate against for the test datapoint
             prediction_date = row[DATE_COLUMN_NAME]
-            predicted_value = run_prediction(prediction_date)
+            predicted_value = run_prediction(prediction_date, frequency)
 
             # Save prediction
             actual.append(dataframe.loc[index, VALUE_COLUMN_NAME])
@@ -198,7 +194,7 @@ class Naive(BaseModel):
 
         forecast = []
         for timestamp in forecast_period_list:
-            forecast_value = run_prediction(timestamp)
+            forecast_value = run_prediction(timestamp, frequency)
             forecast.append(forecast_value)
             new_row = pd.DataFrame({DATE_COLUMN_NAME: [timestamp], "yhat": [forecast_value]})
             dataframe = pd.concat([dataframe, new_row]).reset_index(drop=True)
@@ -221,6 +217,7 @@ class XGBoost(BaseModel):
     def __init__(self):
         """Initializes xgboost model"""
         super().__init__()
+        self.__name__ = "XGBoost"
 
     def create_time_series_features(self, time_series_df):
         result_df = pd.DataFrame(time_series_df[[VALUE_COLUMN_NAME]])
@@ -316,6 +313,7 @@ class RandomForest(BaseModel):
     def __init__(self):
         """Initializes random forest model"""
         super().__init__()
+        self.__name__ = "RandomForest"
 
     def create_time_series_features(self, time_series_df):
         result_df = pd.DataFrame(time_series_df[[VALUE_COLUMN_NAME]])
@@ -413,41 +411,17 @@ class SARIMA(BaseModel):
     def __init__(self):
         """Initializes SARIMA model"""
         super().__init__()
-
-    def step_forecast(self, train_df: pd.DataFrame):
-        # fit a SARIMA model and make a one step prediction
-        train_y = train_df[VALUE_COLUMN_NAME]
-
-        # fit model
-        model = SARIMAX(train_y)
-        model = model.fit(disp=False)
-
-        # make a one-step prediction
-        yhat = model.forecast()
-
-        return yhat.iloc[0]
-
-    def full_forecast(self, train_df, forecast_period_list):
-        # make a one-step prediction
-        forecast = pd.Series()
-        # # Step over each time period and get result
-        for period in forecast_period_list:
-            # fit model on history and make a prediction
-            yhat = self.step_forecast(train_df)
-
-            # store forecast in list of predictions
-            forecast = forecast.append(pd.Series(index=[period], data=[yhat]))
-
-            # add actual observation to history for the next loop
-            train_df = pd.concat([train_df, pd.DataFrame({DATE_COLUMN_NAME: [period], VALUE_COLUMN_NAME: [yhat]})])
-
-        return forecast
+        self.__name__ = "SARIMA"
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
         dataframe = dataframe.copy()
         # Prepare dataframe
         dataframe.set_index("period", inplace=True)
         dataframe = dataframe.asfreq(frequency)
+        if frequency == "W-MON":
+            m = 52
+        else:
+            m = 12
 
         predicted = []
         # Split into train and test
@@ -457,18 +431,22 @@ class SARIMA(BaseModel):
         # Step over each time period and get result
         for index, row in test.iterrows():
             # fit model on history and make a prediction
-            yhat = self.step_forecast(train)
+            model = SARIMAX(train, order=(1, 1, 2), seasonal_order=(1, 1, 2, m))
+            results = model.fit(method='powell')
 
             # store forecast in list of predictions
-            predicted.append(yhat)
+            predicted.append(results.forecast(steps=1))
 
             # add actual observation to history for the next loop
             train = pd.concat([train, pd.DataFrame(row).T])
 
-        # Run full forecast
-        forecast_predictions = self.full_forecast(dataframe, forecast_period_list)
+        model = SARIMAX(train, order=(1, 1, 2), seasonal_order=(1, 1, 2, m))
+        results = model.fit(method='powell')
+        predictions = results.forecast(steps=len(forecast_period_list))
 
-        return test[VALUE_COLUMN_NAME], predicted, forecast_predictions
+        forecast = pd.Series(index=forecast_period_list, data=predictions)
+
+        return test[VALUE_COLUMN_NAME], predicted, forecast
 
 
 def initialize_model(model_name: str):
